<?php

use Battles\Database\DBPDO;
use Battles\Template;

require_once("config.php");
const OK_MAIL_SENT = 'Письмо отправлено!';
const OK_PASSWORD_CHANGED = 'Пароль изменён!';
const ERROR_MAIL_NOT_SENT = 'Письмо не отправлено!';
const ERROR_WRONG_LOGIN = 'Такого пользователя не существует!';
const ERROR_TOO_MANY_TRIES = 'Вы уже отправляли себе письмо сегодня!';
const ERROR_OLD_HASH = 'Ссылка устарела!';
const ERROR_WRONG_HASH = 'Неверная ссылка!';
$login = filter_input(INPUT_POST, 'loginid', FILTER_SANITIZE_SPECIAL_CHARS);
$password = isset($_POST['psw']) ? password_hash($_POST['psw'], PASSWORD_DEFAULT) : null;
$_GET['change'] = $_GET['change'] ?? null;
$newPassword = $_POST['newpasswd'] ?? 0;
$hashCheck = $_POST['hashcheck'] ?? 0;

$operation = new class {
    private function mailSend(string $to, string $message): bool
    {
        $from = "=?UTF-8?B?" . base64_encode('Noreply') . "?= <noreply@" . GAMEDOMAIN . ">";
        $subject = "=?UTF-8?B?" . base64_encode('Восстановление забытого пароля') . "?=";
        $headers = [
            'From' => $from,
            'MIME-Version' => '1.0',
            'Content-type' => 'text/html; charset=UTF-8',
        ];

        if (extension_loaded('tidy')) {
            $cleaner = new tidy();
            $message = $cleaner->repairString($message, ['show-errors' => 0, 'show-warnings' => false], 'utf8');
        }
        return mail($to, $subject, $message, $headers);
    }

    public function sendRecoveryMail(string $to): string
    {
        $db = new DBPDO();
        $check = $db->ofetch('SELECT email FROM users WHERE login = ?', $to);
        if (!$check) {
            return ERROR_WRONG_LOGIN;
        }
        if ($db->ofetch('SELECT 1 FROM users_recovery WHERE login = ?', $to)) {
            return ERROR_TOO_MANY_TRIES;
        }
        $hash = bin2hex(random_bytes(8));
        $db->execute('INSERT INTO users_recovery (login, hash, ip, date) VALUES (?,?,?,?)', [$to, $hash, date('Y-m-d', strtotime('+1days')), $_SERVER['REMOTE_ADDR']]);
        $message = "Здравствуйте!<br><br>
                    Кто-то запросил восстановление пароля к вашему персонажу " . $to . ".<br><br>
                    Для смены пароля пройдите по 
                    <a href='" . GAMEDOMAIN . "/rememberpassword.php?change=" . $hash . "'> данной ссылке</a>.<br><br>
                    Ссылка будет действовать до <em>" . date('d-M-Y', strtotime(date('Y-m-d', strtotime('+1days')))) . "</em>.
                    ";
        return self::mailSend($check->email, $message) ? OK_MAIL_SENT : ERROR_MAIL_NOT_SENT;
    }

    public function isAllowed($hash)
    {
        return DBPDO::INIT()->fetch('SELECT 1 FROM users_recovery WHERE hash = ? AND date < ?', [$hash, date('Y-m-d')]) ? true : ERROR_OLD_HASH;
    }

    public function setNewPassword(string $newPassword, string $hash):string
    {
        $db = new DBPDO();
        $row = $db->ofetch('SELECT login FROM users_recovery WHERE hash = ?', $hash);
        if (!$row) {
            return ERROR_WRONG_HASH;
        }
        $newPassword = password_hash($newPassword, PASSWORD_DEFAULT);
        $db->execute('UPDATE users SET pass = ? WHERE login = ?', [$newPassword, $row->login]);
        $db->execute('DELETE FROM users_recovery WHERE hash = ?', $hash);
        return OK_PASSWORD_CHANGED;
    }
};

if ($login) {
    $statusMessage = $operation->sendRecoveryMail($login);
}

if ($newPassword && $hashCheck) {
    $statusMessage = $operation->setNewPassword($newPassword, $hashCheck);
}
Template::header('Восстановление пароля');
?>
    <a href="/"> ← на главную</a>
<?php if (!empty($statusMessage)): ?>
    <h3><?php echo $statusMessage; ?></h3>
<?php endif; ?>
<?php if ($operation->isAllowed($_GET['change']) === true): ?>
    <h1>Создайте себе новый пароль</h1>
    <form method="post">
        <label>
            <input required name="newpasswd" placeholder="Новый пароль"> Новый пароль.
        </label>
        <input type="hidden" name="hashcheck" value="<?= $_GET['change'] ?>"><br>
        <input type=submit>
    </form>
<?php else: ?>
    <h1>Восстановление пароля</h1>
    <p>Для восстановления пароля введите свой login и нажмите кнопку &laquo;Отправить письмо&raquo;.<br>
        Письмо будет выслано на email адрес, указанный вами при регистрации.<br>
        Восстанавливать пароль можно только раз в сутки.</p>
    <form method="post">
        <label>
            <input required name="loginid" placeholder="Имя персонажа" value="<?= $login ?>"> Имя персонажа.
        </label><br>
        <input type=submit>
    </form>
<?php endif;